; --------------------------------------------------  May 13, 2004
;|               "AGM LCD v1.1"                     |
;|  TI82 assembler LCD line drawing routines (Ash)  |
;|          File name: "agmlcd11.asm"               |
; --------------------------------------------------
;    -----------------------------------------
;   |Author: Jakob Selbing                    |
;   |  Comments, bugs, tips, anything on      |
;   |  your mind:                             |
;   | Email: jakse638@student.liu.se          |
;   | URL: hem.passagen.se/jaksel             |
;   |                                         |
;    -----------------------------------------
;
;
;This source is a simple TI82 program that utilises two routines:
;
; agm_line_bc_de    - draws a line from (b,c) to (d,e) on the LCD
;
; putpixel_hl       - puts a pixel on the LCD at position (l, h)
;
;
;
; My algorithm uses some kind of iterative approach.
; It uses a few additions and subtractions on each pixel,
; instead of multiplication and division. That's why it's 
; more that 3 times faster than my old algorithm!
; The rate is about 5900 pixels / second.
;
;Note: If you want a routine that writes to the GRAPH_MEM instead, 
;      you can use the "AGM" routine, available at the same place 
;      as this file hopefully.
;
;
; YOU CAN USE THESE ROUTINES FREELY, BUT PLEASE GIVE ME CREDIT. :)

#include "ti82.h"
#include "keys.inc"

agm_line_mem_index  = TEXT_MEM      ; Memory base for variables
agm_line_k1         = 0
agm_line_k2         = 1
agm_line_A          = 2
agm_line_status     = 3

agm_line_code_XY_switch = 1


.org START_ADDR
.db "AGM LCD v1.1 test",0


program_start:
    ROM_CALL(CLEARLCD)
    ld b, 0
    ld c, 63
    ld d, 95
    ld e, 63
testloop1:
    push bc
    push de
    call agm_line_bc_de
    pop de
    pop bc
    dec e
    ld a, e
    cp 255
    jr nz, testloop1
    
    ROM_CALL(CLEARLCD)
    ld b, 0
    ld c, 63
    ld d, 95
    ld e, 0
testloop2:
    push bc
    push de
    call agm_line_bc_de
    pop de
    pop bc
    inc b
    ld a, b
    cp 96
    jr nz, testloop2

    
    ROM_CALL(CLEARLCD)
    ld b, 95
    ld c, 63
    ld d, 95
    ld e, 0
testloop3:
    push bc
    push de
    call agm_line_bc_de
    pop de
    pop bc
    dec d
    ld a, d
    cp 255
    jr nz, testloop3

    ROM_CALL(CLEARLCD)
    ld b, 95
    ld c, 63
    ld d, 0
    ld e, 0
testloop4:
    push bc
    push de
    call agm_line_bc_de
    pop de
    pop bc
    dec c
    ld a, c
    cp 255
    jr nz, testloop4
    
Key_loop:
    call GET_KEY
    or a
    jr z, Key_loop
    ret





agm_line_bc_de:
; Draws a line from (b, c) to (d, e) DIRECTLY to the LCD.
; Affects registers: A, B, C, D, E, H, L
;
; b = x1, c = y1
; d = x2, e = y2
;
;        x (0..95)
;    -------->
;   |
;   |
;   |  TI82 screen
;   |
;   \/
;y (0..63)

    ld a, d
    sub b                   ; a = delta X
    jr nc, agm_line_longer_axis_skip_neg1
    neg    
agm_line_longer_axis_skip_neg1:
    ld h, a                 ; h = abs(delta X)
    ld a, e
    sub c                   ; a = delta Y
    jr nc, agm_line_longer_axis_skip_neg2
    neg
agm_line_longer_axis_skip_neg2:
    sub h                   ; a= abs(delta Y) - abs(delta X)
    ld a, 0                 ; Reset "switch XY" flag
    jr c, agm_line_longer_axis_x
    ld a, b                 ; switch X and Y
    ld b, c
    ld c, a
    ld a, d                 ; switch X and Y
    ld d, e
    ld e, a
    ld a, agm_line_code_XY_switch                 ; Set "switch XY" flag
agm_line_longer_axis_x:
    ld (agm_line_mem_index + agm_line_status), a       ; Save status
    
    ld a, b
    cp d
    jr c, agm_line_skip_exchange
    ld a, b                 ; Exchange bc and de
    ld b, d                 ; since algorith assumes b < d
    ld d, a
    ld a, c
    ld c, e
    ld e, a
agm_line_skip_exchange:

    ld a, d
    sub b
    ld (agm_line_mem_index + agm_line_k1), a        ; Save constant K1 = x2 - x1 (>= 0 always)
    ld a, e
    sub c
    ld (agm_line_mem_index + agm_line_k2), a        ; Save constant K2 = y2 - y1
    sub a                   ; A = 0
    ld (agm_line_mem_index + agm_line_A), a     ; Save variable A

agm_line_bc_de_loop:
    push bc
    ld a, (agm_line_mem_index + agm_line_status)
    cp agm_line_code_XY_switch
    jr nz, agm_line_no_XY_switch
    ld a, b                 ; Switch X and Y if this was done earlier
    ld b, c
    ld c, a
agm_line_no_XY_switch:
    ld h, c
    ld l, b
    call putpixel_hl
    pop bc    
        
    ld a, b
    cp d                    ; Check if whole line done
    ret z                   ; Yes, quit
 
    inc b                   ; No, step forward along longer axis
    push de
    push bc

    ld a, (agm_line_mem_index + agm_line_A)     ; Load iterative value "A", which
    ld l, a                             ; should be kept as close to zero as possible
    ld h, 0
    bit 7, a
    jr z, agm_line_A_not_negative
    ld h, 255                           ; Extend to 16-bit 2s complement
agm_line_A_not_negative:

    ld a, (agm_line_mem_index + agm_line_k2)    ; Load constant K2
    ld e, a
    ld d, 0                             ; Extend to 16-bit 2s complement
    bit 7, a
    jr z, agm_line_K2_pos1
    ld d, 255
agm_line_K2_pos1:
    or a                                    ; Clear Carry flag
    sbc hl, de                              ; hl = Aold - K2 = A

    push hl
    
    ld a, (agm_line_mem_index + agm_line_k1)
    ld d, 0
    ld e, a
    ld a, (agm_line_mem_index + agm_line_k2)                    ; If delta Y negative then 
    bit 7, a                                ; we will test A - K1 instead of A + K1
    jr z, agm_line_K2_pos2
    or a
    sbc hl, de                              ; hl = A - K1
    jr agm_line_ex_de_hl
agm_line_K2_pos2:
    add hl, de                              ; hl = A + K1
agm_line_ex_de_hl:
    ex de, hl                               ; de = A +/- K1 = B
    
    pop hl                                  ; hl = A
    push hl
    push de
        
    bit 7, h
    jr z, agm_line_A_abs_done
    ld a, l
    neg
    ld l, a
    ld h, 0
agm_line_A_abs_done:
    bit 7, d
    jr z, agm_line_B_abs_done
    ld a, e
    neg
    ld e, a
    ld d, 0
agm_line_B_abs_done:
    or a                                    ; Clear Carry flag
    sbc hl, de                              ; hl = abs(A) - abs(B)
    pop de                                  ; pop B = A +/- K1
    pop hl                                  ; pop A
    pop bc                                  ; pop position 1
    jr c, agm_line_keep_y                       ; If abs(A) < abs(A +/- K1) then Y should not increase/decrease
    ld a, (agm_line_mem_index + agm_line_k2)
    bit 7, a
    jr nz, agm_line_dec_y
    inc c                                   ; Increase Y position
    inc c
agm_line_dec_y:
    dec c
    ld l, e                                 ; Make code below save (A +/- K1) instead of A
agm_line_keep_y:
    ld a, l                                 ; Keep Y position
    ld (agm_line_mem_index + agm_line_A), a         ; A' = A
    pop de                                  ; Pop position 2
    jp agm_line_bc_de_loop    


        
putpixel_hl:
; This routine turns on a pixel at (l,h).
; Affects registers: A, B, H, L
;
; l = x
; h = y
;
;        l (0..95)
;    -------->
;   |
;   |
;   |  TI82 screen
;   |
;   \/
;h (0..63)
    ld a,l
    and %00000111
    ld b,a
    srl l
    srl l
    srl l
    ld a,l
    or $20
    out (P_LCD_CON),a
    ld l,a
    inc b
    sub a
    scf
shift_pixel_right
    rra
    djnz shift_pixel_right
    ld c, a
    ld a,$80
    or h
    ld h,a
    or 0             ; extra delay for the LCD controller
    out (P_LCD_CON),a
    call DISP_DELAY
    in a,(P_LCD_DATA)
    call DISP_DELAY
    in a,(P_LCD_DATA)
    or c
    ld c,a
    ld a,l
    call DISP_DELAY
    out (P_LCD_CON),a
    ld a,h
    call DISP_DELAY
    out (P_LCD_CON),a
    ld a,c
    call DISP_DELAY
    out (P_LCD_DATA),a
    ret   



.end



